@@ -1,3 +1,18 @@
+# Copyright (C) 2020  Andrii Kolomoiets <andreyk.mad@gmail.com>
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
+
 import logging
 import re
 
@@ -27,7 +42,7 @@ from pygls.features import (COMPLETION, TEXT_DOCUMENT_DID_CHANGE,
                             REFERENCES, WORKSPACE_DID_CHANGE_CONFIGURATION,
                             TEXT_DOCUMENT_WILL_SAVE, TEXT_DOCUMENT_DID_SAVE,
                             DOCUMENT_SYMBOL, CODE_ACTION, FORMATTING,
-                            RANGE_FORMATTING)
+                            RANGE_FORMATTING, RENAME, DOCUMENT_HIGHLIGHT)
 from pygls import types
 from pygls.server import LanguageServer
 from pygls.protocol import LanguageServerProtocol
@@ -45,7 +60,8 @@ _COMPLETION_TYPES = {
     'function': types.CompletionItemKind.Function,
     'param': types.CompletionItemKind.Variable,
     'keyword': types.CompletionItemKind.Keyword,
-    'statement': types.CompletionItemKind.Variable
+    'statement': types.CompletionItemKind.Variable,
+    'property': types.CompletionItemKind.Property
 }
 
 
@@ -345,7 +361,7 @@ def _validate(ls: LanguageServer, uri: str):
                 types.Position(x.line - 1, x.column),
                 types.Position(x.until_line - 1, x.until_column)
             ),
-            'Invalid syntax',
+            x.get_message(),
             types.DiagnosticSeverity.Error,
             source='jedi'
         )
@@ -434,6 +450,8 @@ def _completions_snippets(completions: List[Completion],
             sort_text=_completion_sort_key(completion, completionPrefixPlain),
             **item
         )
+        if completion.type == 'property':
+            continue
         for signature in completion.get_signatures():
             names = []
             snippets = []
@@ -556,14 +574,18 @@ def signature_help(
     return None
 
 
+def _get_name_range(name: Name) -> types.Range:
+    return types.Range(
+        types.Position(name.line - 1, name.column),
+        types.Position(name.line - 1, name.column + len(name.name))
+    )
+
+
 def _get_locations(defs: List[Name]) -> List[types.Location]:
     return [
         types.Location(
             from_fs_path(d.module_path),
-            types.Range(
-                types.Position(d.line - 1, d.column),
-                types.Position(d.line - 1, d.column + len(d.name))
-            )
+            _get_name_range(d)
         )
         for d in defs if d.module_path
     ]
@@ -836,3 +858,37 @@ def range_formatting(
         ls: LanguageServer, params: types.DocumentRangeFormattingParams
 ) -> Optional[List[types.TextEdit]]:
     return _formatting(ls, params.textDocument.uri, params.range)
+
+
+@server.feature(RENAME)
+def rename(ls: LanguageServer,
+           params: types.RenameParams) -> Optional[types.WorkspaceEdit]:
+    script = get_script(ls, params.textDocument.uri)
+    try:
+        refactoring = script.rename(params.position.line + 1,
+                                    params.position.character,
+                                    new_name=params.newName)
+    except RefactoringError:
+        return None
+    document_changes = _get_document_changes(ls, refactoring)
+    if document_changes:
+        return types.WorkspaceEdit(document_changes=document_changes)
+    return None
+
+
+@server.feature(DOCUMENT_HIGHLIGHT)
+def highlight(
+        ls: LanguageServer, params: types.TextDocumentPositionParams
+) -> Optional[List[types.DocumentHighlight]]:
+    script = get_script(ls, params.textDocument.uri)
+    names = script.get_references(params.position.line + 1,
+                                  params.position.character,
+                                  scope='file')
+    if not names:
+        return None
+    return [
+        types.DocumentHighlight(
+            _get_name_range(name)
+        )
+        for name in names
+    ]
